/*
 * kdfwidget.cpp
 *
 * Copyright (c) 1998 Michael Kropfberger mkropfbe@edu.uni-klu.ac.at
 *
 * Requires the Qt widget libraries, available at no cost at
 * http://www.troll.no/
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
#include <math.h>

#include <qstring.h>
#include <qmsgbox.h> 
#include <qfile.h>
#include <qtstream.h>
#include <qstring.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpixmap.h>
#include <qpaintd.h>

#include <kapp.h> 
#include <kmsgbox.h> 
#include <kiconloader.h>
#include "kdfwidget.h"
#include "kdfwidget.moc"


#define BLANK ' '
#define DELIMITER '#'
#define BORDER 5
#define PIX_COLUMN 6
#define FULL_PERCENT 95.0
#define VISIBLE klocale->translate("visible")
#define INVISIBLE klocale->translate("invisible")

#define DEFAULT_FREQ 60
#define DEFAULT_FILEMGR_COMMAND "kfmclient openURL %p"
//#define DEFAULT_FILEMGR_COMMAND "kvt -e mc %p"

static bool GUI;


/***************************************************************************
  * Constructor
**/
KDFWidget::KDFWidget (QWidget * parent, const char *name, bool init)
    : KConfigWidget (parent, name)
{
  //debug("Construct: KDFWidget::KDFWidget");

#ifdef _OS_LINUX_
  debug("_OS_LINUX_");
#endif
#ifdef _OS_FREEBSD_
  debug("_OS_FREEBSD_");
#endif
#ifdef _OS_SUN_
  debug("_OS_SUN_");
#endif
#ifdef _OS_SOLARIS_
  debug("_OS_SOLARIS_");
#endif
#ifdef _OS_HPUX_
  debug("_OS_HPUX_");
#endif
#ifdef _OS_UNIX_
  debug("_OS_UNIX_");
#endif

if (NO_FS_TYPE)
  debug("df gives no FS_TYPE");

  popupIfFull=TRUE;
  updateFreq=DEFAULT_FREQ;

  tabWidths.resize(PIX_COLUMN+1);
  tabHeaders.append(klocale->translate("Device") );
  tabWidths[0]=80;
  tabHeaders.append(klocale->translate("Type") );
  tabWidths[1]=50;
  tabHeaders.append(klocale->translate("Size") );
  tabWidths[2]=50;
  tabHeaders.append(klocale->translate("MountPoint") );
  tabWidths[3]=80;
  tabHeaders.append(klocale->translate("Free") );
  tabWidths[4]=50;
  tabHeaders.append(klocale->translate("Full%") );
  tabWidths[5]=50;
  tabHeaders.append(klocale->translate("UsageBar") );
  tabWidths[6]=100;

  if (init) {
    GUI = FALSE;
  } else
    GUI = TRUE;

  if (GUI)
    {  // inits go here
      this->setMinimumSize(323,200);
      KIconLoader *loader = kapp->getIconLoader();


      settingsBox = new QGroupBox(klocale->translate("Settings"),this);
      CHECK_PTR(settingsBox);
      freqLabel = new QLabel( klocale->translate("update frequency (seconds)")
                              ,settingsBox);
      fileMgrLabel = new QLabel( klocale->translate("FileManager (e.g. kvt -e mc %p)")
                              ,settingsBox);
      freqScroll = new QScrollBar( settingsBox );
      CHECK_PTR(freqScroll); 
      freqScroll->setOrientation( QScrollBar::Horizontal );
      freqScroll->setSteps(1,20);
      freqScroll->setRange(0, 180 ); 
      freqLCD = new QLCDNumber( settingsBox );
      CHECK_PTR(freqLCD);
      freqLCD->setNumDigits( 3 );
      freqLCD->setSegmentStyle(QLCDNumber::Filled);
      connect(freqScroll, SIGNAL(valueChanged(int)), 
              freqLCD, SLOT(display(int)) );
      connect(freqScroll, SIGNAL(valueChanged(int)), 
              this, SLOT(setUpdateFreq(int)) );
      freqScroll->setValue(updateFreq);  // defaultValue
      fileMgrEdit = new QLineEdit( settingsBox );
      
      showSettings=FALSE;
      settingsBtn=new QPushButton(klocale->translate("Settings"),settingsBox);
      connect(settingsBtn,SIGNAL(clicked()),this,SLOT(settingsBtnClicked()));
      settingsBtn->setToggleButton(TRUE);
      defaultsBtn=new QPushButton(klocale->translate("Defaults"),settingsBox);
      connect(defaultsBtn,SIGNAL(clicked()),this,SLOT(defaultsBtnClicked()));

      QString pcn=parent->className();
      //debug("parent: [%s]",pcn.data());
      if ( pcn == "KDFTopLevel" ) {
         //it is used from KTopLevelWidget "kdf" --> provides a help-btn
         isTopLevel=TRUE;  
      } else 
         isTopLevel=FALSE;

      if (isTopLevel) {
        helpBtn=new QPushButton(klocale->translate("Help"),settingsBox);
        connect(helpBtn,SIGNAL(clicked()),this,SLOT(invokeHTMLHelp()));
     } else
        helpBtn=0; //if

	 
   // normal tabList ****************************************

      tabList = new KTabListBox(this,"tabList",PIX_COLUMN+1,0); 
      CHECK_PTR(tabList);
      for (int i=0;i<PIX_COLUMN;i++)
        tabList->setColumn(i,tabHeaders.at(i)
                         ,tabWidths[i],KTabListBox::TextColumn);
      tabList->setColumn(PIX_COLUMN,tabHeaders.at(PIX_COLUMN)
                         ,tabWidths[PIX_COLUMN]
                         ,KTabListBox::PixmapColumn);
			 
      tabList->setSeparator(BLANK);
      connect(tabList,SIGNAL(headerClicked(int))
             ,this,SLOT(headerClicked(int)) );
      connect(tabList,SIGNAL(popupMenu(int,int))
             ,this,SLOT(popupMenu(int,int)) );
      connect(tabList,SIGNAL(selected(int,int))
             ,this,SLOT(popupMenu(int,int)) );
      connect(tabList,SIGNAL(midClick(int,int))
             ,this,SLOT(popupMenu(int,int)) );
      connect(tabList,SIGNAL(highlighted(int,int))
           ,this,SLOT(popupMenu(int,int)) );
      tabList->dict().setAutoDelete(TRUE);


   // config-tabList ****************************************

      confTabList = new KTabListBox(settingsBox,"confTabList",PIX_COLUMN+1,0); 
      CHECK_PTR(confTabList);
      for (int i=0;i<=PIX_COLUMN;i++)
        confTabList->setColumn(i,tabHeaders.at(i)
                         ,tabWidths[i],KTabListBox::PixmapColumn);
      confTabList->setSeparator(BLANK);
      confTabList->dict().setAutoDelete(TRUE);
      QPixmap *pix;
      pix = new QPixmap(loader->loadIcon("tick.xpm"));
      confTabList->dict().replace(VISIBLE,pix );
      pix = new QPixmap(loader->loadIcon("filedel.xpm"));
      confTabList->dict().replace(INVISIBLE,pix );
    
      QString s;
      for (int i=0;i<=PIX_COLUMN;i++)
         s = s + klocale->translate(VISIBLE) + " ";
      confTabList->appendItem((const char *)s);
      connect(confTabList,SIGNAL(headerClicked(int))
             ,this,SLOT(toggleColumnVisibility(int)) );
      connect(confTabList,SIGNAL(popupMenu(int,int))
             ,this,SLOT(toggleColumnVisibility(int,int)) );
      connect(confTabList,SIGNAL(selected(int,int))
             ,this,SLOT(toggleColumnVisibility(int,int)) );
      connect(confTabList,SIGNAL(midClick(int,int))
             ,this,SLOT(toggleColumnVisibility(int,int)) );
      connect(confTabList,SIGNAL(highlighted(int,int)) 
           ,this,SLOT(toggleColumnVisibility(int,int)) );
 
      diskList.setAutoDelete(TRUE);

   // BackgroundProcesses ****************************************

      sysProc = new KProcess(); CHECK_PTR(sysProc);
       connect( sysProc, SIGNAL(receivedStdout(KProcess *, char *, int) ),
           this, SLOT (receivedSysStdErrOut(KProcess *, char *, int)) );
       connect( sysProc, SIGNAL(receivedStderr(KProcess *, char *, int) ),
           this, SLOT (receivedSysStdErrOut(KProcess *, char *, int)) );
      connect(sysProc,SIGNAL(processExited(KProcess *) ),
              this, SLOT(sysCallDone() ) );
	     
      dfProc = new KProcess(); CHECK_PTR(dfProc);
       connect( dfProc, SIGNAL(receivedStdout(KProcess *, char *, int) ),
           this, SLOT (receivedDFStdErrOut(KProcess *, char *, int)) );
       connect( dfProc, SIGNAL(receivedStderr(KProcess *, char *, int) ),
           this, SLOT (receivedDFStdErrOut(KProcess *, char *, int)) );
       connect(dfProc,SIGNAL(processExited(KProcess *) ),
              this, SLOT(dfDone() ) );

       readingDFStdErrOut=FALSE;
       readingSysStdErrOut=FALSE;
       readFSTAB();
       readDF();
    }//if GUI

   config = kapp->getConfig();
   loadSettings();
   if (init)
     applySettings();
} // Constructor


/***************************************************************************
  * Destructor
**/
KDFWidget::~KDFWidget() 
{ 
  //debug("DESTRUCT: KDFWidget::~KDFWidget");
  if (GUI) {
  }
}; 

void KDFWidget::closeEvent(QCloseEvent *)
{
  applySettings();
  kapp->quit();
};

/***************************************************************************
  * saves KConfig and starts the df-cmd
**/
void KDFWidget::applySettings()
{
  //debug("KDFWidget::applySettings");
 config->setGroup("KDiskFree");
 if (isTopLevel) { // only "kdf" is resizable
   config->writeEntry("Width",width() );
   config->writeEntry("Height",height() );
 }
 config->writeEntry("UpdateFrequency",updateFreq);
 if (GUI) {
    config->writeEntry("FileManagerCommand",fileMgrEdit->text() );
    for (int i=0;i<=PIX_COLUMN;i++)
       config->writeEntry(tabHeaders.at(i),tabList->columnWidth(i) );
 } else { //init
    config->writeEntry("FileManagerCommand",DEFAULT_FILEMGR_COMMAND );
    for (int i=0;i<=PIX_COLUMN;i++)
       config->writeEntry(tabHeaders.at(i),tabWidths[i] );
 }//if
 config->sync();

 if (GUI) { 
   readFSTAB();
   readDF();
 }
}


/***************************************************************************
  * reads the KConfig
**/
void KDFWidget::loadSettings()
{
  //debug("KDFWidget::loadSettings");
 config->setGroup("KDiskFree");
 updateFreq=config->readNumEntry("UpdateFrequency",DEFAULT_FREQ);
 if (GUI) {
    if (isTopLevel) { //only "kdf" can be resized
       int appWidth=config->readNumEntry("Width",width());
       int appHeight=config->readNumEntry("Height",height());
       this->setGeometry(0,0,appWidth,appHeight);
    }
    freqScroll->setValue(updateFreq);
    fileMgrEdit->setText(config->readEntry("FileManagerCommand"
                         ,DEFAULT_FILEMGR_COMMAND));
    for (int i=0;i<=PIX_COLUMN;i++) {
       tabList->setColumnWidth(i,config->readNumEntry(tabHeaders.at(i)
                                                  ,tabWidths[i]) );
    if (tabList->columnWidth(i) != 0)
       confTabList->changeItemPart(klocale->translate(VISIBLE),0,i); 
    else
       confTabList->changeItemPart(klocale->translate(INVISIBLE),0,i); 
   
    }//for
  readFSTAB();
  readDF();
 }//if GUI
}

/***************************************************************************
  * resets all to its defaults
**/
void KDFWidget::defaultsBtnClicked()
{
  //debug("KDFWidget::defaultsBtnClicked");
  freqScroll->setValue(DEFAULT_FREQ);
  fileMgrEdit->setText(DEFAULT_FILEMGR_COMMAND);
  for (int i=0;i<=PIX_COLUMN;i++) {
       tabList->setColumnWidth(i,tabWidths[i]);
    if (tabList->columnWidth(i) != 0)
       confTabList->changeItemPart(klocale->translate(VISIBLE),0,i); 
    else
       confTabList->changeItemPart(klocale->translate(INVISIBLE),0,i); 
  }//for
  this->update();
  readFSTAB();
  readDF();
}

/***************************************************************************
  * pops up the SettingsBox if the settingsBtn is clicked
**/
void KDFWidget::settingsBtnClicked()
{
  if (showSettings)
     showSettings=FALSE;
  else
     showSettings=TRUE;
  applySettings();
  repaint();
}


/***************************************************************************
  * resets the timer for automatic df-refreshes
**/
void KDFWidget::setUpdateFreq(int freq)
{
  //debug("KDFWidget::setUpdateFreq");
  killTimers(); //kills !all! running timers
  updateFreq=freq;
  if (updateFreq > 0)  //0 sets to NO_AUTO_UPDATE   ;)
    startTimer( updateFreq * 1000 );
}

/***************************************************************************
  * Update (reread) all disk-dependencies
**/
void KDFWidget::timerEvent(QTimerEvent *) 
{ 
  //debug("KDFWidget::timerEvent");
  if (readingDFStdErrOut || dfProc->isRunning() ) { 
    //warning("dfProc timed out... (maybe unreachable hard NFS-mounts)");
     return;  //timeout
  }//if
  readFSTAB();
  readDF();
};


/***************************************************************************
  * tries to figure out the possibly mounted fs
**/
void KDFWidget::readFSTAB()
{
  //debug("KDFWidget::readFSTAB");
  if (readingDFStdErrOut) return;
  QFile f(FSTAB);
  if ( f.open(IO_ReadOnly) ) {
    QTextStream t (&f);
    QString s;
    diskList.clear();
    tabList->setAutoUpdate(FALSE);
    tabList->clear();
    DiskEntry *disk;
    while (! t.eof()) {
      s=t.readLine();
      s=s.simplifyWhiteSpace();
      if ( (!s.isEmpty() ) && (s.find(DELIMITER)!=0) ) {
               // not empty or commented out by '#'
	//debug("GOT: [%s]",s.data() );
         disk = new DiskEntry(this); CHECK_PTR(disk);
         disk->setMounted(FALSE);
         disk->setDeviceName(s.left(s.find(BLANK)) );
            s=s.remove(0,s.find(BLANK)+1 );
	    //debug("    deviceName:    [%s]",(const char*)disk->deviceName() );
#ifdef _OS_SOLARIS_
            //device to fsck
            s=s.remove(0,s.find(BLANK)+1 );
#endif        
         disk->setMountPoint(s.left(s.find(BLANK)) );
            s=s.remove(0,s.find(BLANK)+1 );
	    //debug("    MountPoint:    [%s]",(const char*)disk->mountPoint() );
         disk->setFsType(s.left(s.find(BLANK)) );
            s=s.remove(0,s.find(BLANK)+1 );
	    //debug("    FS-Type:       [%s]",(const char*)disk->fsType() );
         disk->setMountOptions(s.left(s.find(BLANK)) );
            s=s.remove(0,s.find(BLANK)+1 );
	    //debug("    Mount-Options: [%s]",(const char*)disk->mountOptions() );

         if ( (disk->deviceName() != "none") && (disk->fsType() != "swap") 
            && (disk->mountPoint() != "/dev/swap")
            && (disk->deviceName() != "/proc") ) {
	   // exclude swap-partitions and the proc-device   
           replaceDeviceEntry(disk);
           //disk->show();
         } else {
           delete disk;
         } //if not swap or proc or none
      } //if not empty
    } //while
    f.close();
    tabList->setAutoUpdate(TRUE);
  } //if f.open
}

/***************************************************************************
  * reads the df-commands results
**/
void KDFWidget::readDF()
{
  //debug("KDFWidget::readDF");
  if (readingDFStdErrOut || dfProc->isRunning()) return; 
  dfStringErrOut=""; // yet no data received
  dfProc->clearArguments();
  (*dfProc) << DF_COMMAND << DF_ARGS;
  if (!dfProc->start( KProcess::NotifyOnExit, KProcess::AllOutput ))
     fatal(klocale->translate("could not execute [%s]"),DF_COMMAND);
}


/***************************************************************************
  * is called, when the df-command has finished
**/
void KDFWidget::dfDone()
{
  //debug("KDFWidget::dfDone");
  readingDFStdErrOut=TRUE;
  tabList->setAutoUpdate(FALSE);
  for ( DiskEntry *disk=diskList.first(); disk != 0; disk=diskList.next() ) 
    disk->setMounted(FALSE);  // set all devs unmounted

  for (unsigned int i=0; i<tabList->count(); i++) {
    tabList->changeItemPart("",i,PIX_COLUMN-2); //unmounted devs have no Free
    tabList->changeItemPart("",i,PIX_COLUMN-1); //unmounted devs have no Free%
    tabList->changeItemPart("",i,PIX_COLUMN); //unmounted devs have no pixBar
  }//for

  QTextStream t (dfStringErrOut, IO_ReadOnly);
  QString s=t.readLine();
  if ( (s.isEmpty()) || ( s.left(10) != "Filesystem" ) ) 
    fatal("Error running df command... got [%s]",(const char *)s);
  while ( !t.eof() ) {
    QString u,v;
    DiskEntry *disk;
    s=t.readLine();
    s=s.simplifyWhiteSpace();
    if ( !s.isEmpty() ) {
      //debug("GOT: [%s]",s.data() );
      disk = new DiskEntry(this); CHECK_PTR(disk);
      
      if (s.find(BLANK)<0)      // devicename was too long, rest in next line
	if ( !t.eof() ) {       // just appends the next line
            v=t.readLine();
            s=s.append(v.data() );
            s=s.simplifyWhiteSpace();
            //debug("SPECIAL GOT: [%s]",s.data() );
	 }//if silly linefeed

      //debug("EFFECTIVELY GOT %i chars: [%s]",s.length(),s.data() );
	 
      disk->setDeviceName(s.left(s.find(BLANK)) );
      s=s.remove(0,s.find(BLANK)+1 );
      //debug("    DeviceName:    [%s]",(const char*)disk->deviceName() );

      if (NO_FS_TYPE) {
	//debug("THERE IS NO FS_TYPE_FIELD!");
         disk->setFsType("?");
      } else {
         disk->setFsType(s.left(s.find(BLANK)) );
         s=s.remove(0,s.find(BLANK)+1 );
      };
      //debug("    FS-Type:       [%s]",(const char*)disk->fsType() );

      u=s.left(s.find(BLANK));
      disk->setKBSize(u.toInt() );
      s=s.remove(0,s.find(BLANK)+1 );
      //debug("    Size:       [%d]",disk->kBSize() );

      u=s.left(s.find(BLANK));
      disk->setKBUsed(u.toInt() );
      s=s.remove(0,s.find(BLANK)+1 );
      //debug("    Used:       [%d]",disk->kBUsed() );
    
      u=s.left(s.find(BLANK));
      disk->setKBAvail(u.toInt() );
      s=s.remove(0,s.find(BLANK)+1 );
      //debug("    Avail:       [%d]",disk->kBAvail() );
    

      s=s.remove(0,s.find(BLANK)+1 );  // delete the capacity 94%
      disk->setMountPoint(s.left(s.find(BLANK)) );
      s=s.remove(0,s.find(BLANK)+1 );
      //debug("    MountPoint:       [%s]",(const char*)disk->mountPoint() );
  
      if (disk->kBSize() > 0) {
        disk->setMounted(TRUE);    // its now mounted (df lists only mounted)
        replaceDeviceEntry(disk);
      }
    }//if not header
  }//while further lines available

  tabList->setAutoUpdate(TRUE);
  this->update();  //enables the automatic PIX_COLUMN-resize but flickers!
  updatePixmaps();
  readingDFStdErrOut=FALSE;
}

/***************************************************************************
  * updates or creates a new DiskEntry in the KDFList and TabListBox
**/
void KDFWidget::replaceDeviceEntry(DiskEntry *disk)
{
  //debug("KDFWidget::replaceDeviceEntry");
  QString percS="?";
  QString sizeS="?";
  if ( disk->kBSize() > 0 ) {
    percS.sprintf("%2.1f%%",disk->percentFull() );
    sizeS=disk->prettyKBSize();
  }
  else {
    sizeS=klocale->translate("UNKNOWN");    
    percS=klocale->translate("UNKNOWN");    
  }

  int pos=diskList.find(disk);

  if ((pos == -1) && (disk->mounted()) )
    // no matching entry found for mounted disk
    if ((disk->fsType() == "?") || (disk->fsType() == "cachefs")) {
      //search for fitting cachefs-entry in static /etc/vfstab-data
      //debug("search for [%s]",(const char *)disk->deviceName());
      DiskEntry* olddisk = diskList.first();
      QString odiskName;
      while (olddisk != 0) {
        int p;
        // cachefs deviceNames have no / behind the host-column
	// eg. /cache/cache/.cfs_mnt_points/srv:_home_jesus
	//                                      ^    ^
        odiskName = olddisk->deviceName().copy();
        int ci=odiskName.find(':'); // goto host-column
        while ((ci =odiskName.find('/',ci)) > 0) {
           odiskName.replace(ci,1,"_");
        }//while
        //debug("     check against [%s]",(const char *)olddisk->deviceName());
        // check if there is something that is exactly the tail
	// eg. [srv:/tmp3] is exact tail of [/cache/.cfs_mnt_points/srv:_tmp3]
        if ( ( (p=disk->deviceName().findRev(odiskName
	            ,disk->deviceName().length()) )
                != -1) 
	      && (p + odiskName.length()
	          == disk->deviceName().length()) ) 
        {
             pos = diskList.at(); //store the actual position
	     //debug("   FOUND [%s] at pos %i at ListPos %i"
	     //  ,(const char *)olddisk->deviceName(),p,pos);
             disk->setDeviceName(olddisk->deviceName());
             olddisk=0;
	} else 
          olddisk=diskList.next();       
      }// while
    }// if fsType == "?" or "cachefs"
  

#ifdef NO_FS_TYPE
  if (pos != -1) {
     DiskEntry * olddisk = diskList.at(pos);
     if (olddisk)
        disk->setFsType(olddisk->fsType());     
  }
#endif  
  QString s;
  s.sprintf("%s %s %s %s %s %s %s"
                          ,(const char *)disk->deviceName()
                          ,(const char *)disk->fsType()
                          ,(const char *)sizeS
                          ,(const char *)disk->mountPoint()
                          ,(const char *)disk->prettyKBAvail()
                          ,(const char *)percS
                          ,(const char *)disk->deviceName() );
  //debug("store [%s]", (const char *)s);
  if (pos != -1) {  // replace
    unsigned int ipos=0;

    QString e1, e2;
    while(ipos < tabList->count()) {  //replace data of old entry
       e1=tabList->text(ipos,0);
       e2=tabList->text(ipos,3);

       if (e1==(const char *)disk->deviceName()
	  && (e2==(const char *)disk->mountPoint())) {
             //entry found
          DiskEntry * olddisk = diskList.at(pos);
          //debug("GOT %f vs. %f",olddisk->percentFull(),disk->percentFull());
          if ((popupIfFull) &&
               (olddisk->percentFull()<FULL_PERCENT) && 
	        (disk->percentFull()>FULL_PERCENT)) {
            QString st;
            st.sprintf("Device %s is getting critically full!"
                        ,(const char *)disk->deviceName());
            QMessageBox::warning(0,klocale->translate("WARNING")
                 ,(const char*)st);
	  }
          diskList.remove(pos); // delete old one
          diskList.insert(pos,disk);
          tabList->changeItem(s,ipos);
          ipos=tabList->count(); // exit forced
       }//if
       ipos++;
    }//while     
  } else {
    diskList.append(disk);
    tabList->appendItem((const char *)s);
  }//if 

}

/***************************************************************************
  * is called, when the df-command writes on StdOut or StdErr
**/
void KDFWidget::receivedDFStdErrOut(KProcess *, char *data, int len)
{
  //debug("KDFWidget::receivedDFStdErrOut");
  QString tmp = QString(data,len+1);  // adds a zero-byte 
  dfStringErrOut.append(tmp);
}

/***************************************************************************
  * is called, when the Sys-command writes on StdOut or StdErr
**/
void KDFWidget::receivedSysStdErrOut(KProcess *, char *data, int len)
{
  //debug("KDFWidget::receivedSysStdErrOut");
  QString tmp = QString(data,len+1);  // adds a zero-byte 
  sysStringErrOut.append(tmp);
}


/***************************************************************************
  * starts a command on the underlying system via /bin/sh
**/
void KDFWidget::sysCall(QString command)
{
  //debug("KDFWidget::sysCall");
  if (readingSysStdErrOut || sysProc->isRunning()) {
    //debug("sysProc running or reading StdErrOut!");
   return;
  }

  sysStringErrOut=""; // yet no data received
  sysProc->clearArguments();
  (*sysProc) << "/bin/sh" << "-c" << (const char *)command;
  if (!sysProc->start( KProcess::NotifyOnExit, KProcess::AllOutput ))
      fatal(klocale->translate("could not execute [%s]"),(const char *)command);
}

/**************************************************************************
  * is called when the sysCall finishes
**/
void KDFWidget::sysCallDone()
{
  //debug("KDFWidget::sysCallDone [%s]",(const char *)sysStringErrOut);
  readingSysStdErrOut=TRUE;

  if (!sysStringErrOut.isEmpty() ) 
     QMessageBox::critical(this,klocale->translate("ERROR")
                           ,(const char *)sysStringErrOut);

  //tabList->unmarkAll();
  readFSTAB();
  readDF();
  readingSysStdErrOut=FALSE;
}

/**************************************************************************
  * connected with the tabList-Signal
**/
void KDFWidget::headerClicked(int )  //ignore column
{
  //debug("KDFWidget::headerClicked: %d",column);
}

/**************************************************************************
  * connected with the confTabList-Signal
**/
void KDFWidget::toggleColumnVisibility(int column)
{
  //debug("KDFWidget::toggleColumnVisibility: %d %d",column,tabList->columnWidth(column));

  if(tabList->columnWidth(column) == 0) {
    tabList->setColumnWidth(column,tabWidths[column]);
    confTabList->changeItemPart(klocale->translate(VISIBLE),0,column); 
  } else {
    tabList->setColumnWidth(column,0);
    confTabList->changeItemPart(klocale->translate(INVISIBLE),0,column); 
  }
  //confTabList->unmarkAll();
  this->update();
  readFSTAB();
  readDF();
}

/**************************************************************************
  * pops up and asks for mount/umount right-clicked device
**/
void KDFWidget::popupMenu(int row,int column)
{
  //debug("KDFWidget::popupMenu: %d:%d",row,column);

  // get the clicked disk
      QString dev=tabList->text(row,0);
      QString mnt=tabList->text(row,3);
      DiskEntry *disk = new DiskEntry(dev);
      disk->setMountPoint(mnt);
      int pos=diskList.find(disk);
      delete disk;
      disk=diskList.at(pos);
  
      unsigned int ipos=0;
      QString e1, e2;
      while(ipos<tabList->count()) {  //find old entry in tabList
         e1=tabList->text(ipos,0);
	 e2=tabList->text(ipos,3);
	 if (e1==(const char *)disk->deviceName()
	  && (e2==(const char *)disk->mountPoint())) {
                                //entry found
             tabList->markItem(ipos);
            tabList->changeItemColor(yellow,ipos);
         }//if
         ipos++;
      }//while        
 
  
  QString cmdS;
  switch (column) { 
  case 0:   //mount/umount deviceName  
      if (disk->mounted())
       cmdS.sprintf("%s %s",UMOUNT_COMMAND,(const char *)disk->deviceName());
      else
       cmdS.sprintf("%s %s",MOUNT_COMMAND,(const char *)disk->deviceName());
      /*
        cmdS.sprintf("%s %s -t%s %s -o%s",MOUNT_COMMAND
                            ,(const char *)disk->deviceName()
                            ,(const char *)disk->fsType()
                            ,(const char *)disk->mountPoint()
                            ,(const char *)disk->mountOptions()  );
       */

      //debug("mount-cmd: [%s]",(const char *)cmdS);
      sysCall(cmdS);
      break;
  case 3:  //open fileManager on MountPoint
      if ( fileMgrEdit->text() != "" ) {
         cmdS=fileMgrEdit->text();
         int pos=cmdS.find("%p");
         if ( pos > 0 ) {
             cmdS=cmdS.replace(pos,2
                           ,(const char *)disk->mountPoint() );
             cmdS.append(" &");
         } else //no %p found; just add path on the back
             cmdS=cmdS+" "+(const char *)disk->mountPoint()+" &";
         //debug("fileMgr-cmd: [%s]",(const char *)cmdS);
  
	system(cmdS);
	//sysCall(cmdS);
       }//if 
      break;
 }//switch

}

/**************************************************************************
  * recalculates and repaints the pixBars
**/
void KDFWidget::updatePixmaps()
{
  //debug("KDFWidget::updatePixmaps");
  tabList->setAutoUpdate(FALSE);
  tabList->unmarkAll();
  for ( DiskEntry *disk=diskList.first(); disk != 0; disk=diskList.next() ) {
    if ( !disk->mounted() ) { 
       unsigned int ipos=0;
       QString e1, e2;
       while(ipos<tabList->count()) {  //find old entry in tabList
          e1=tabList->text(ipos,0);
	  e2=tabList->text(ipos,3);
	  if ((e1==(const char *)disk->deviceName())
	    && (e2==(const char *)disk->mountPoint())) {
              //entry found
	    //tabList->markItem(ipos);
             tabList->changeItemColor(lightGray,ipos);
             //tabList->dict().remove((const char *)disk->deviceName());
          }//if
        ipos++;
        }//while        
    }//if not mounted
    else {
      if (disk->percentFull() > FULL_PERCENT) {
       unsigned int ipos=0;
       QString e1, e2;
       while(ipos<tabList->count()) {  //find old entry in tabList
          e1=tabList->text(ipos,0);
	  e2=tabList->text(ipos,3);
	  if ((e1==(const char *)disk->deviceName()) 
	    && (e2==(const char *)disk->mountPoint())) {
                                    //entry found
	    //tabList->markItem(ipos);
             tabList->changeItemColor(red,ipos);
          }//if
        ipos++;
        }//while        
      }//if full over FULL_PERCENT
      
      if (tabList->cellWidth(PIX_COLUMN) > 7) {
        QPixmap *pix;
        pix = new QPixmap(tabList->cellWidth(PIX_COLUMN)-7
                      , tabList->cellHeight(1)-1 ); CHECK_PTR(pix);
        if ( !pix->isNull() ) {
           pix->fill(white );
           QPainter p(pix);
           p.setPen(black);
           p.drawRect(0,0,pix->width(),pix->height());
           if (disk->percentFull() > FULL_PERCENT)
              p.setBrush(red);
           else
              p.setBrush(darkBlue);
           p.setPen(white);
           p.drawRect(1,1
              ,(int)(  ((float)pix->width()-2) 
                     * (disk->percentFull()/100) )
              ,pix->height()-2 );
    
           tabList->dict().replace((const char *)disk->deviceName(), pix);
        }//if
      }//if usageBar is visible
    }//was mounted
  }//for
  tabList->setAutoUpdate(TRUE);
  tabList->repaint();
  confTabList->unmarkAll();
  confTabList->repaint();
}

/**************************************************************************
  * calculates the sizes and shows all devices
**/
void KDFWidget::paintEvent(QPaintEvent *)
{
  //debug("KDFWidget::paintEvent");

   int tabListHeight=this->height()-settingsBox->height();
   if (!showSettings)
      tabListHeight += freqLCD->y()-3;
   int oldPixColumnWidth=tabList->columnWidth(PIX_COLUMN);
   tabList->setGeometry(0, 0, this->width()-1,tabListHeight);
       //this -1 is necessary....         ^^^^  god knows why?
      //setGeometry also changes the width of last column!!
  tabList->setColumnWidth(PIX_COLUMN,oldPixColumnWidth);
  //debug("PIX_COLUMN_width=%d",tabList->columnWidth(PIX_COLUMN));

  //resizing of PIX_COLUMN *************************************
  if (tabList->columnWidth(PIX_COLUMN) != 0 ){
    int totalWidth=0;
    for (int i=0;i<PIX_COLUMN;i++) //all except the last (pix) column
      totalWidth += tabList->columnWidth(i);
    //adjust size to fit completely into tabList-widget...
    if (tabList->width() > totalWidth+4)
       tabList->setColumnWidth(PIX_COLUMN,tabList->width()-totalWidth-4);
  }//if user hasn't turned of the usageBar

  //set widths to default-values
  for (int i=0;i<=PIX_COLUMN;i++)
      confTabList->setColumnWidth(i,tabWidths[i]);
  //same with confTabList
  int totalWidth=0;
  for (int i=0;i<PIX_COLUMN;i++) //all except the last (pix) column
    totalWidth += confTabList->columnWidth(i);
  //adjust size to fit completely into tabList-widget...
  if (confTabList->width() > totalWidth+4)
     confTabList->setColumnWidth(PIX_COLUMN
                        ,confTabList->width()-totalWidth-4);


  updatePixmaps();
}

/**************************************************************************
  * calculates the sizes of the settings and the device-tabList
**/
void KDFWidget::resizeEvent(QResizeEvent *)
{
  //debug("KDFWidget::resizeEvent  %dx%d",width(),height());
 settingsBox->setGeometry(0,this->height()-170,this->width()-1,170); 

 settingsBtn->setGeometry(BORDER,settingsBox->height()-2*BORDER-32,90,32);
 if (isTopLevel)  //only if executed as "kdf"
   helpBtn->setGeometry(settingsBox->width()-BORDER-settingsBtn->width()
           ,settingsBtn->y(),settingsBtn->width()
           ,settingsBtn->height());

 
 freqLCD->setGeometry(BORDER+settingsBtn->x()+settingsBtn->width()
                ,settingsBox->height()-2*BORDER-32,52,32);
 {
   int x=freqLCD->x()+freqLCD->width()+BORDER;
   int w=160;
   w=settingsBox->width() - x - BORDER;
   if (isTopLevel)
     w -= helpBtn->width() + BORDER;
   freqScroll->setGeometry(x,settingsBox->height()-2*BORDER-16,w,16);
 }
	    
 // confTabList->setGeometry(0,settingsBox->y(),settingsBox->width()-1,60);
 confTabList->setGeometry(BORDER,3*BORDER
                    ,settingsBox->width()-2*BORDER,60);
 
 defaultsBtn->setGeometry(BORDER
                        ,settingsBtn->y()-32-BORDER,90,32);

  freqLabel->setGeometry(freqLCD->x()+freqLCD->width()+BORDER
                  ,freqLCD->y()
		  ,freqScroll->width()+3
                  ,freqLCD->height()-freqScroll->height() );
  fileMgrLabel->setGeometry(defaultsBtn->x()+defaultsBtn->width()+BORDER
                  ,defaultsBtn->y()-7
                  ,freqScroll->width()+freqLCD->width()+BORDER
                  ,16 );
  fileMgrEdit->setGeometry(fileMgrLabel->x()
                 ,(fileMgrLabel->y()+fileMgrLabel->height())
                 ,fileMgrLabel->width()
                 ,(defaultsBtn->y()+defaultsBtn->height())
                   -(fileMgrLabel->y()+fileMgrLabel->height()) );
 repaint();
}

